@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -80,7 +80,8 @@ enum {
 
 #define INFINOTED_PLUGIN_MANAGER_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INFINOTED_TYPE_PLUGIN_MANAGER, InfinotedPluginManagerPrivate))
 
-static GObjectClass* parent_class;
+G_DEFINE_TYPE_WITH_CODE(InfinotedPluginManager, infinoted_plugin_manager, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfinotedPluginManager))
 
 static gpointer
 infinoted_plugin_manager_hash(gpointer first,
@@ -377,11 +378,10 @@ infinoted_plugin_manager_load_plugin(InfinotedPluginManager* manager,
 
   if(module == NULL)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       infinoted_plugin_manager_error_quark(),
       INFINOTED_PLUGIN_MANAGER_ERROR_OPEN_FAILED,
-      "%s",
       g_module_error()
     );
 
@@ -391,11 +391,10 @@ infinoted_plugin_manager_load_plugin(InfinotedPluginManager* manager,
 
   if(g_module_symbol(module, "INFINOTED_PLUGIN", (gpointer*)&plugin) == FALSE)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       infinoted_plugin_manager_error_quark(),
       INFINOTED_PLUGIN_MANAGER_ERROR_NO_ENTRY_POINT,
-      "%s",
       g_module_error()
     );
     
@@ -712,13 +711,9 @@ infinoted_plugin_manager_set_directory(InfinotedPluginManager* manager,
 }
 
 static void
-infinoted_plugin_manager_init(GTypeInstance* instance,
-                              gpointer g_class)
+infinoted_plugin_manager_init(InfinotedPluginManager* manager)
 {
-  InfinotedPluginManager* manager;
   InfinotedPluginManagerPrivate* priv;
-
-  manager = INFINOTED_PLUGIN_MANAGER(instance);
   priv = INFINOTED_PLUGIN_MANAGER_PRIVATE(manager);
 
   priv->directory = NULL;
@@ -762,7 +757,7 @@ infinoted_plugin_manager_dispose(GObject* object)
     priv->credentials = NULL;
   }
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(infinoted_plugin_manager_parent_class)->dispose(object);
 }
 
 static void
@@ -782,7 +777,7 @@ infinoted_plugin_manager_finalize(GObject* object)
 
   g_free(priv->path);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(infinoted_plugin_manager_parent_class)->finalize(object);
 }
 
 static void
@@ -857,14 +852,11 @@ infinoted_plugin_manager_get_property(GObject* object,
 }
 
 static void
-infinoted_plugin_manager_class_init(gpointer g_class,
-                                    gpointer class_data)
+infinoted_plugin_manager_class_init(
+  InfinotedPluginManagerClass* manager_class)
 {
   GObjectClass* object_class;
-  object_class = G_OBJECT_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfinotedPluginManagerPrivate));
+  object_class = G_OBJECT_CLASS(manager_class);
 
   object_class->dispose = infinoted_plugin_manager_dispose;
   object_class->finalize = infinoted_plugin_manager_finalize;
@@ -920,50 +912,19 @@ infinoted_plugin_manager_class_init(gpointer g_class,
   );
 }
 
-GType
-infinoted_plugin_manager_get_type(void)
-{
-  static GType plugin_manager_type = 0;
-
-  if(!plugin_manager_type)
-  {
-    static const GTypeInfo plugin_manager_type_info = {
-      sizeof(InfinotedPluginManagerClass), /* class_size */
-      NULL,                                /* base_init */
-      NULL,                                /* base_finalize */
-      infinoted_plugin_manager_class_init, /* class_init */
-      NULL,                                /* class_finalize */
-      NULL,                                /* class_data */
-      sizeof(InfinotedPluginManager),      /* instance_size */
-      0,                                   /* n_preallocs */
-      infinoted_plugin_manager_init,       /* instance_init */
-      NULL                                 /* value_table */
-    };
-
-    plugin_manager_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfinotedPluginManager",
-      &plugin_manager_type_info,
-      0
-    );
-  }
-
-  return plugin_manager_type;
-}
-
 /**
- * infinoted_plugin_manager_new:
+ * infinoted_plugin_manager_new: (constructor)
  * @directory: The #InfdDirectory on which plugins should operate.
  * @log: The #InfinotedLog to write log messages to.
- * @creds: The #InfCertificateCredentials used to secure data transfer with
- * the clients, or %NULL.
+ * @creds: (allow-none): The #InfCertificateCredentials used to secure data
+ * transfer with the clients, or %NULL.
  *
  * Creates a new #InfinotedPluginManager with the given directory, log
  * and credentials. These three objects will be available for plugins
  * to enhance the infinoted functionality. Plugins can be loaded
  * with infinoted_plugin_manager_load().
  *
- * Returns: A new #InfinotedPluginManager.
+ * Returns: (transfer full): A new #InfinotedPluginManager.
  */
 InfinotedPluginManager*
 infinoted_plugin_manager_new(InfdDirectory* directory,
@@ -990,7 +951,8 @@ infinoted_plugin_manager_new(InfdDirectory* directory,
  * infinoted_plugin_manager_load:
  * @manager: A #InfinotedPluginManager.
  * @plugin_path: The path from which to load plugins.
- * @plugins: A list of plugins to load, or %NULL.
+ * @plugins: (array zero-terminated=1) (allow-none): A list of plugins to
+ * load, or %NULL.
  * @options: A #GKeyFile with configuration options for the plugins.
  * @error: Location to store error information, if any, or %NULL.
  *
@@ -1071,7 +1033,7 @@ infinoted_plugin_manager_load(InfinotedPluginManager* manager,
  *
  * Returns the #InfdDirectory used by the plugin manager.
  *
- * Returns: A #InfdDirectory owned by the plugin manager.
+ * Returns: (transfer none): A #InfdDirectory owned by the plugin manager.
  */
 InfdDirectory*
 infinoted_plugin_manager_get_directory(InfinotedPluginManager* manager)
@@ -1086,7 +1048,7 @@ infinoted_plugin_manager_get_directory(InfinotedPluginManager* manager)
  *
  * Returns the #InfIo of the #InfdDirectory used by the plugin manager.
  *
- * Returns: A #InfIo owned by the plugin manager.
+ * Returns: (transfer none): A #InfIo owned by the plugin manager.
  */
 InfIo*
 infinoted_plugin_manager_get_io(InfinotedPluginManager* manager)
@@ -1106,7 +1068,7 @@ infinoted_plugin_manager_get_io(InfinotedPluginManager* manager)
  * Returns the #InfinotedLog that the plugin manager and the plugins do
  * write log messages to.
  *
- * Returns: A #InfinotedLog owned by the plugin manager.
+ * Returns: (transfer none): A #InfinotedLog owned by the plugin manager.
  */
 InfinotedLog*
 infinoted_plugin_manager_get_log(InfinotedPluginManager* manager)
@@ -1122,7 +1084,8 @@ infinoted_plugin_manager_get_log(InfinotedPluginManager* manager)
  * Returns the #InfCertificateCredentials used for securing the data transfer
  * with all clients.
  *
- * Returns: A #InfCertificateCredentials object owned by the plugin manager.
+ * Returns: (transfer none): A #InfCertificateCredentials object owned by the
+ * plugin manager.
  */
 InfCertificateCredentials*
 infinoted_plugin_manager_get_credentials(InfinotedPluginManager* manager)
@@ -1154,7 +1117,8 @@ infinoted_plugin_manager_error_quark(void)
  * @plugin_info. Returns %NULL if no such object exists, i.e. when the
  * plugin's @connection_info_size is set to 0.
  *
- * Returns: A pointer to the connection-specific plugin data, or %NULL.
+ * Returns: (transfer none) (allow-none): A pointer to the connection-specific
+ * plugin data, or %NULL.
  */
 gpointer
 infinoted_plugin_manager_get_connection_info(InfinotedPluginManager* mgr,
@@ -1184,7 +1148,8 @@ infinoted_plugin_manager_get_connection_info(InfinotedPluginManager* mgr,
  * @plugin_info. Returns %NULL if no such object exists, i.e. when the
  * plugin's @session_info_size is set to 0.
  *
- * Returns: A pointer to the session-specific plugin data, or %NULL.
+ * Returns: (transfer none) (allow-none): A pointer to the session-specific
+ * plugin data, or %NULL.
  */
 gpointer
 infinoted_plugin_manager_get_session_info(InfinotedPluginManager* mgr,
