@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -57,7 +57,7 @@ struct AvahiWatch {
   InfIoWatch* watch;
 
   InfNativeSocket socket;
-  AvahiWatchEvent occured_events;
+  AvahiWatchEvent occurred_events;
 
   AvahiWatchCallback callback;
   void* userdata;
@@ -73,7 +73,6 @@ struct AvahiTimeout {
 typedef struct _InfDiscoveryAvahiInfoResolv InfDiscoveryAvahiInfoResolv;
 struct _InfDiscoveryAvahiInfoResolv {
   InfDiscoveryResolvCompleteFunc complete_func;
-  InfDiscoveryResolvErrorFunc error_func;
   gpointer user_data;
 };
 
@@ -114,6 +113,8 @@ struct _InfDiscoveryAvahiPrivate {
   InfIo* io;
   InfXmppManager* xmpp_manager;
   InfXmppConnectionSecurityPolicy security_policy;
+  InfKeepalive keepalive;
+
   InfCertificateCredentials* creds;
   InfSaslContext* sasl_context;
   gchar* sasl_mechanisms;
@@ -130,19 +131,26 @@ enum {
   /* construct only */
   PROP_XMPP_MANAGER,
   PROP_IO,
+
+  /* read/write */
   PROP_CREDENTIALS,
   PROP_SASL_CONTEXT,
   PROP_SASL_MECHANISMS,
-
-  /* read/write */
-  PROP_SECURITY_POLICY
+  PROP_SECURITY_POLICY,
+  PROP_KEEPALIVE
 };
 
 #define INF_DISCOVERY_AVAHI_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_TYPE_DISCOVERY_AVAHI, InfDiscoveryAvahiPrivate))
 
-static GObjectClass* parent_class;
 static GQuark inf_discovery_avahi_error_quark;
 
+static void inf_discovery_avahi_discovery_iface_init(InfDiscoveryInterface* iface);
+static void inf_discovery_avahi_local_publisher_iface_init(InfLocalPublisherInterface* iface);
+G_DEFINE_TYPE_WITH_CODE(InfDiscoveryAvahi, inf_discovery_avahi, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfDiscoveryAvahi)
+  G_IMPLEMENT_INTERFACE(INF_TYPE_DISCOVERY, inf_discovery_avahi_discovery_iface_init)
+  G_IMPLEMENT_INTERFACE(INF_TYPE_LOCAL_PUBLISHER, inf_discovery_avahi_local_publisher_iface_init))
+
 /*
  * Destroy notification callbacks
  */
@@ -174,6 +182,7 @@ inf_discovery_avahi_info_resolv_complete(InfDiscoveryInfo* info)
     resolv->complete_func(
       info,
       INF_XML_CONNECTION(info->resolved),
+      NULL,
       resolv->user_data
     );
 
@@ -195,7 +204,7 @@ inf_discovery_avahi_info_resolv_error(InfDiscoveryInfo* info,
   {
     resolv = (InfDiscoveryAvahiInfoResolv*)item->data;
 
-    resolv->error_func(info, error, resolv->user_data);
+    resolv->complete_func(info, NULL, error, resolv->user_data);
     g_slice_free(InfDiscoveryAvahiInfoResolv, resolv);
   }
 
@@ -348,10 +357,12 @@ inf_discovery_avahi_service_resolver_callback(AvahiServiceResolver* resolver,
       g_object_set(
         G_OBJECT(tcp),
         "device-index", discovery_info->interface,
-        NULL);
+        NULL
+      );
 
       error = NULL;
-      if(inf_tcp_connection_open(tcp, &error) == FALSE)
+      if(!inf_tcp_connection_set_keepalive(tcp, &priv->keepalive, &error) ||
+         !inf_tcp_connection_open(tcp, &error))
       {
         inf_discovery_avahi_info_resolv_error(discovery_info, error);
         g_error_free(error);
@@ -432,11 +443,10 @@ inf_discovery_avahi_service_resolver_callback(AvahiServiceResolver* resolver,
   case AVAHI_RESOLVER_FAILURE:
     error = NULL;
 
-    g_set_error(
+    g_set_error_literal(
       &error,
       inf_discovery_avahi_error_quark,
       avahi_client_errno(avahi_service_resolver_get_client(resolver)),
-      "%s",
       avahi_strerror(
         avahi_client_errno(avahi_service_resolver_get_client(resolver))
       )
@@ -871,7 +881,7 @@ inf_discovery_avahi_client_callback(AvahiClient* client,
     {
       g_warning(
         _("Avahi client is in failure state. Service discovery or "
-          "publishing is no longer possible.\n\nThe occured failure "
+          "publishing is no longer possible.\n\nThe occurred failure "
           "was: %s\n"),
         avahi_strerror(avahi_client_errno(client))
       );
@@ -923,8 +933,8 @@ inf_discovery_avahi_watch_cb(InfNativeSocket* socket,
   AvahiWatch* watch;
   watch = (AvahiWatch*)user_data;
 
-  watch->occured_events = inf_discovery_avahi_from_io_event(event);
-  watch->callback(watch, *socket, watch->occured_events, watch->userdata);
+  watch->occurred_events = inf_discovery_avahi_from_io_event(event);
+  watch->callback(watch, *socket, watch->occurred_events, watch->userdata);
 }
 
 static void
@@ -954,7 +964,7 @@ inf_discovery_avahi_watch_new(const AvahiPoll* api,
 
   watch->avahi = avahi;
   watch->socket = fd;
-  watch->occured_events = 0;
+  watch->occurred_events = 0;
   watch->callback = callback;
   watch->userdata = userdata;
 
@@ -987,7 +997,7 @@ inf_discovery_avahi_watch_update(AvahiWatch* watch,
 static AvahiWatchEvent
 inf_discovery_avahi_watch_get_events(AvahiWatch* watch)
 {
-  return watch->occured_events;
+  return watch->occurred_events;
 }
 
 static void
@@ -1089,13 +1099,9 @@ inf_discovery_avahi_timeout_free(AvahiTimeout* timeout)
  */
 
 static void
-inf_discovery_avahi_init(GTypeInstance* instance,
-                         gpointer g_class)
+inf_discovery_avahi_init(InfDiscoveryAvahi* avahi)
 {
-  InfDiscoveryAvahi* avahi;
   InfDiscoveryAvahiPrivate* priv;
-
-  avahi = INF_DISCOVERY_AVAHI(instance);
   priv = INF_DISCOVERY_AVAHI_PRIVATE(avahi);
   
   priv->poll.userdata = avahi;
@@ -1110,6 +1116,7 @@ inf_discovery_avahi_init(GTypeInstance* instance,
   priv->io = NULL;
   priv->xmpp_manager = NULL;
   priv->security_policy = INF_XMPP_CONNECTION_SECURITY_BOTH_PREFER_TLS;
+  priv->keepalive.mask = 0;
   priv->creds = NULL;
   priv->sasl_context = NULL;
   priv->sasl_mechanisms = NULL;
@@ -1125,19 +1132,12 @@ inf_discovery_avahi_init(GTypeInstance* instance,
   );
 }
 
-static GObject*
-inf_discovery_avahi_constructor(GType type,
-                                guint n_construct_properties,
-                                GObjectConstructParam* construct_properties)
+static void
+inf_discovery_avahi_constructed(GObject* object)
 {
-  GObject* object;
   InfDiscoveryAvahiPrivate* priv;
   
-  object = G_OBJECT_CLASS(parent_class)->constructor(
-    type,
-    n_construct_properties,
-    construct_properties
-  );
+  G_OBJECT_CLASS(inf_discovery_avahi_parent_class)->constructed(object);
 
   priv = INF_DISCOVERY_AVAHI_PRIVATE(object);
 
@@ -1145,7 +1145,6 @@ inf_discovery_avahi_constructor(GType type,
   g_assert(priv->io != NULL);
 
   inf_discovery_avahi_create_client(INF_DISCOVERY_AVAHI(object));
-  return object;
 }
 
 static void
@@ -1196,7 +1195,7 @@ inf_discovery_avahi_dispose(GObject* object)
     priv->io = NULL;
   }
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_discovery_avahi_parent_class)->dispose(object);
 }
 
 static void
@@ -1210,7 +1209,7 @@ inf_discovery_avahi_finalize(GObject* object)
 
   g_free(priv->sasl_mechanisms);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_discovery_avahi_parent_class)->finalize(object);
 }
 
 static void
@@ -1249,6 +1248,10 @@ inf_discovery_avahi_set_property(GObject* object,
   case PROP_SECURITY_POLICY:
     priv->security_policy = g_value_get_enum(value);
     break;
+  case PROP_KEEPALIVE:
+    g_assert(g_value_get_boxed(value) != NULL);
+    priv->keepalive = *(const InfKeepalive*)g_value_get_boxed(value);
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
     break;
@@ -1287,6 +1290,9 @@ inf_discovery_avahi_get_property(GObject* object,
   case PROP_SECURITY_POLICY:
     g_value_set_enum(value, priv->security_policy);
     break;
+  case PROP_KEEPALIVE:
+    g_value_set_boxed(value, &priv->keepalive);
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
     break;
@@ -1345,7 +1351,6 @@ static void
 inf_discovery_avahi_resolve(InfDiscovery* discovery,
                             InfDiscoveryInfo* info,
                             InfDiscoveryResolvCompleteFunc complete_func,
-                            InfDiscoveryResolvErrorFunc error_func,
                             gpointer user_data)
 {
   InfDiscoveryAvahiPrivate* priv;
@@ -1368,23 +1373,34 @@ inf_discovery_avahi_resolve(InfDiscovery* discovery,
       error = NULL;
       if(!inf_xml_connection_open(INF_XML_CONNECTION(info->resolved), &error))
       {
-        error_func(info, error, user_data);
+        complete_func(info, NULL, error, user_data);
         g_error_free(error);
       }
       else
       {
-        complete_func(info, INF_XML_CONNECTION(info->resolved), user_data);
+        complete_func(
+          info,
+          INF_XML_CONNECTION(info->resolved),
+          NULL,
+          user_data
+        );
       }
 
       break;
     case INF_XML_CONNECTION_CLOSING:
       /* TODO: We should add ourselves to the resolver list, and wait for
        * the connection being closed and reopen it afterwards. */
-      error_func(info, NULL, user_data);
+      complete_func(info, NULL, NULL, user_data);
       break;
     case INF_XML_CONNECTION_OPENING:
     case INF_XML_CONNECTION_OPEN:
-      complete_func(info, INF_XML_CONNECTION(info->resolved), user_data);
+      complete_func(
+        info,
+        INF_XML_CONNECTION(info->resolved),
+        NULL,
+        user_data
+      );
+
       break;
     default:
       g_assert_not_reached();
@@ -1395,7 +1411,6 @@ inf_discovery_avahi_resolve(InfDiscovery* discovery,
   {
     resolv = g_slice_new(InfDiscoveryAvahiInfoResolv);
     resolv->complete_func = complete_func;
-    resolv->error_func = error_func;
     resolv->user_data = user_data;
     info->resolv = g_slist_prepend(info->resolv, resolv);
 
@@ -1422,11 +1437,10 @@ inf_discovery_avahi_resolve(InfDiscovery* discovery,
           avahi_service_resolver_get_client(info->service_resolver)
         );
 
-        g_set_error(
+        g_set_error_literal(
           &error,
           inf_discovery_avahi_error_quark,
           errno,
-          "%s",
           avahi_strerror(errno)
         );
 
@@ -1511,19 +1525,12 @@ inf_discovery_avahi_unpublish(InfLocalPublisher* publisher,
  */
 
 static void
-inf_discovery_avahi_class_init(gpointer g_class,
-                               gpointer class_data)
+inf_discovery_avahi_class_init(InfDiscoveryAvahiClass* avahi_class)
 {
   GObjectClass* object_class;
-  InfDiscoveryAvahiClass* avahi_class;
+  object_class = G_OBJECT_CLASS(avahi_class);
 
-  object_class = G_OBJECT_CLASS(g_class);
-  avahi_class = INF_DISCOVERY_AVAHI_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfDiscoveryAvahiPrivate));
-
-  object_class->constructor = inf_discovery_avahi_constructor;
+  object_class->constructed = inf_discovery_avahi_constructed;
   object_class->dispose = inf_discovery_avahi_dispose;
   object_class->finalize = inf_discovery_avahi_finalize;
   object_class->set_property = inf_discovery_avahi_set_property;
@@ -1565,7 +1572,7 @@ inf_discovery_avahi_class_init(gpointer g_class,
       "Certificate credentials",
       "The GnuTLS certificate credentials used for encrypting XMPP streams",
       INF_TYPE_CERTIFICATE_CREDENTIALS,
-      G_PARAM_READWRITE | G_PARAM_CONSTRUCT
+      G_PARAM_READWRITE
     )
   );
 
@@ -1577,7 +1584,7 @@ inf_discovery_avahi_class_init(gpointer g_class,
       "SASL context",
       "The SASL context used for authentication",
       INF_TYPE_SASL_CONTEXT,
-      G_PARAM_READWRITE | G_PARAM_CONSTRUCT
+      G_PARAM_READWRITE
     )
   );
 
@@ -1589,7 +1596,7 @@ inf_discovery_avahi_class_init(gpointer g_class,
       "SASL mechanisms",
       "The accepted SASL mechanisms for authentication",
       NULL,
-      G_PARAM_READWRITE | G_PARAM_CONSTRUCT
+      G_PARAM_READWRITE
     )
   );
 
@@ -1605,15 +1612,23 @@ inf_discovery_avahi_class_init(gpointer g_class,
       G_PARAM_READWRITE
     )
   );
+
+  g_object_class_install_property(
+    object_class,
+    PROP_KEEPALIVE,
+    g_param_spec_boxed(
+      "keepalive",
+      "Keepalive",
+      "The keepalive settings for new connections",
+      INF_TYPE_KEEPALIVE,
+      G_PARAM_READWRITE
+    )
+  );
 }
 
 static void
-inf_discovery_avahi_discovery_init(gpointer g_iface,
-                                   gpointer iface_data)
+inf_discovery_avahi_discovery_iface_init(InfDiscoveryInterface* iface)
 {
-  InfDiscoveryIface* iface;
-  iface = (InfDiscoveryIface*)g_iface;
-
   iface->discover = inf_discovery_avahi_discover;
   iface->get_discovered = inf_discovery_avahi_get_discovered;
   iface->resolve = inf_discovery_avahi_resolve;
@@ -1624,77 +1639,19 @@ inf_discovery_avahi_discovery_init(gpointer g_iface,
 }
 
 static void
-inf_discovery_avahi_local_publisher_init(gpointer g_iface,
-                                         gpointer iface_data)
+inf_discovery_avahi_local_publisher_iface_init(
+  InfLocalPublisherInterface* iface)
 {
-  InfLocalPublisherIface* iface;
-  iface = (InfLocalPublisherIface*)g_iface;
-
   iface->publish = inf_discovery_avahi_publish;
   iface->unpublish = inf_discovery_avahi_unpublish;
 }
 
-GType
-inf_discovery_avahi_get_type(void)
-{
-  static GType discovery_avahi_type = 0;
-
-  if(!discovery_avahi_type)
-  {
-    static const GTypeInfo discovery_avahi_type_info = {
-      sizeof(InfDiscoveryAvahiClass),  /* class_size */
-      NULL,                            /* base_init */
-      NULL,                            /* base_finalize */
-      inf_discovery_avahi_class_init,  /* class_init */
-      NULL,                            /* class_finalize */
-      NULL,                            /* class_data */
-      sizeof(InfDiscoveryAvahi),       /* instance_size */
-      0,                               /* n_preallocs */
-      inf_discovery_avahi_init,        /* instance_init */
-      NULL                             /* value_table */
-    };
-
-    static const GInterfaceInfo discovery_info = {
-      inf_discovery_avahi_discovery_init,
-      NULL,
-      NULL
-    };
-
-    static const GInterfaceInfo local_publisher_info = {
-      inf_discovery_avahi_local_publisher_init,
-      NULL,
-      NULL
-    };
-
-    discovery_avahi_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfDiscoveryAvahi",
-      &discovery_avahi_type_info,
-      0
-    );
-
-    g_type_add_interface_static(
-      discovery_avahi_type,
-      INF_TYPE_DISCOVERY,
-      &discovery_info
-    );
-
-    g_type_add_interface_static(
-      discovery_avahi_type,
-      INF_TYPE_LOCAL_PUBLISHER,
-      &local_publisher_info
-    );
-  }
-
-  return discovery_avahi_type;
-}
-
 /*
  * Public API.
  */
 
 /**
- * inf_discovery_avahi_new:
+ * inf_discovery_avahi_new: (constructor)
  * @io: A #InfIo object used for watching sockets and timeouts.
  * @manager: A #InfXmppManager.
  * @creds: The certificate credentials used for GnuTLS encryption.
@@ -1719,7 +1676,7 @@ inf_discovery_avahi_get_type(void)
  * the server. It can be %NULL, in which case all available mechanisms are
  * accepted.
  *
- * Return Value: A new #InfDiscoveryAvahi.
+ * Return Value: (transfer full): A new #InfDiscoveryAvahi.
  **/
 InfDiscoveryAvahi*
 inf_discovery_avahi_new(InfIo* io,
@@ -1783,6 +1740,39 @@ inf_discovery_avahi_get_security_policy(InfDiscoveryAvahi* discovery)
   return INF_DISCOVERY_AVAHI_PRIVATE(discovery)->security_policy;
 }
 
+/**
+ * inf_discovery_avahi_set_keepalive:
+ * @discovery: A #InfDiscoveryAvahi.
+ * @keepalive: The new keepalive configuration.
+ *
+ * Sets the keepalive settings for newly created connections. It does not
+ * affect already existing connections.
+ */
+void
+inf_discovery_avahi_set_keepalive(InfDiscoveryAvahi* discovery,
+                                  const InfKeepalive* keepalive)
+{
+  g_return_if_fail(INF_IS_DISCOVERY_AVAHI(discovery));
+  g_return_if_fail(keepalive != NULL);
+
+  INF_DISCOVERY_AVAHI_PRIVATE(discovery)->keepalive = *keepalive;
+}
+
+/**
+ * inf_discovery_avahi_get_keepalive:
+ * @discovery: A #InfDiscoveryAvahi.
+ *
+ * Returns the keepalive settings that will be used for new connections.
+ *
+ * Returns: A #InfKeepalive set for new connections, owned by @discovery.
+ */
+const InfKeepalive*
+inf_discovery_avahi_get_keepalive(InfDiscoveryAvahi* discovery)
+{
+  g_return_val_if_fail(INF_IS_DISCOVERY_AVAHI(discovery), NULL);
+  return &INF_DISCOVERY_AVAHI_PRIVATE(discovery)->keepalive;
+}
+
 #endif /* LIBINFINITY_HAVE_AVAHI */
 
 /* vim:set et sw=2 ts=2: */
